/*
 * This unfortunate adaptor lets an srf client use an sr object by throwing
 * if the object actually sends non-zero flags.  I really would like to
 * always make this check at compile time, but I simply can't figure out
 * how to do it without introducing mutiple class names that I don't want
 * to have.
 */

/*
 * This file is part of the Cleansocks library.  It is released under the
 * Lesser GNU GPL.  See the accompanying files COPYING and COPYING.LESSER.
 */

#ifndef _cleansrf2sr_h_
#define _cleansrf2sr_h_

#include "cleansri.h"

namespace cleansocks {

class srf2sr: public srf_obj {
public:
	srf2sr(): m_srf(NULL) { }
	srf2sr(sr_obj &srf): m_srf(&srf) { }
	void operator()(sr_obj &srf) { m_srf = &srf; }
	virtual int send(const void *buf, int size) {
		if(!m_srf)
			throw socket_interface_error
				("Uninitialized srwrapper object.");
		return cleansocks::send(*m_srf,buf,size);
	}
	virtual int send(const void *buf, int size, int flags) {
		if(flags != 0)
			throw socket_interface_error
				("Interface does not handle flags");
		return cleansocks::send(*m_srf,buf,size);
	}
	virtual int recv(void *buf, int size) {
		if(!m_srf)
			throw socket_interface_error
				("Uninitialized srwrapper object.");
		return cleansocks::recv(*m_srf,buf,size);
	}
	virtual int recv(void *buf, int size, int flags) {
		if(flags != 0)
			throw socket_interface_error
				("Interface does not handle flags");
		return cleansocks::recv(*m_srf,buf,size);
	}

	virtual void close() { 
		if(!m_srf)
			throw socket_interface_error
				("Uninitialized srwrapper object.");
		cleansocks::close(*m_srf);
	}
	
	virtual void shutdownsend() {
		throw socket_interface_error
			("Interface does not support shutdown");
	}
	virtual void shutdownrecv() {
		throw socket_interface_error
			("Interface does not support shutdown");
	}


	bool valid() { return m_srf != NULL; }
private:
	sr_obj *m_srf;
};

}
#endif
